bl_info = {
    "name": "My Tools",
    "author": "YOURNAME",
    "version": (0, 2, 0),
    "blender": (2, 76, 2),
    "location": "",
    "description": "",
    "warning": "",
    "wiki_url": "",
    "category": "Object"
}

import bpy
from bpy.props import *
import math
from bpy.types import Operator, Panel, PropertyGroup


class VIEW3D_OT_MyTools(Operator):
    bl_idname = "object.my_tools"
    bl_label = "My Tools"
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        return True if context.mode == 'OBJECT' else False

    def execute(self, context):

        scene = bpy.context.scene
        settings = scene.my_tool_settings

        objects = context.selected_objects if settings.selected_only else scene.objects

        if settings.object_type == 'ALL_TYPES':
            if not settings.selected_only: # No point reselecting as we used context.selected_objects
                for obj in objects:
                    obj.select = True
        else:
            for obj in objects:
                obj.select = True if obj.type == settings.object_type else False

        bpy.ops.object.origin_set(type=settings.origin_type)
        bpy.ops.object.transform_apply(location=settings.location, rotation=settings.rotation, scale=settings.scale)
        bpy.ops.object.shade_smooth()

        mesh_objects = [ob for ob in objects if ob.type == 'MESH']

        for ob in mesh_objects:
            mesh = ob.data
            mesh.use_auto_smooth = True
            mesh.auto_smooth_angle = settings.autosmooth_angle
            mesh.show_normal_face = True

            scene.objects.active = ob # Make this object the active one
            bpy.ops.object.mode_set(mode='EDIT')
            bpy.ops.mesh.normals_make_consistent()
            bpy.ops.object.mode_set(mode='OBJECT')

        return {'FINISHED'}


class PG_MySetings(PropertyGroup):

    ob_types = [('ALL_TYPES', "All Types", "", 1),
             ('MESH', "Meshes Only", "", 2),
             ('CURVE', "Curves Only", "", 3)]

    # Filter
    object_type = EnumProperty(items=ob_types, name="Object Types", default='ALL_TYPES', options={'SKIP_SAVE'})
    selected_only = BoolProperty(name="Selected Only", 
                                 description="Apply only to the selected object(s) otherwise apply to \
                                              all scene objects of the specified object type.",
                                 default=True)

    origin_types = [('GEOMETRY_ORIGIN', "GEOMETRY_ORIGIN", "", 1),
                    ('ORIGIN_GEOMETRY', "ORIGIN_GEOMETRY", "", 2),
                    ('ORIGIN_CURSOR', "ORIGIN_CURSOR", "", 3),
                    ('ORIGIN_CENTER_OF_MASS', "ORIGIN_CENTER_OF_MASS", "", 4)]

    origin_type = EnumProperty(items=origin_types, 
                               name="Origin Set To", 
                               default="ORIGIN_GEOMETRY")

    location = BoolProperty(name="Location", default=False)
    rotation = BoolProperty(name="Rotation", default=True)
    scale = BoolProperty(name="Scale", default=True)

    autosmooth_angle = FloatProperty(name="Auto Smooth Angle", 
                                     subtype='ANGLE', 
                                     default=math.radians(37.0))


class VIEW3D_PT_MyTools(Panel):

    #bl_idname
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'

    bl_context = "objectmode"
    bl_category = "Tools"
    bl_label = "My Tools"

    def draw(self, context):
        
        settings = context.scene.my_tool_settings
        
        layout = self.layout
        col = layout.column()

        col.operator("object.my_tools", text="Run")
        col.prop(settings, "object_type")
        col.prop(settings, "selected_only")
        col.prop(settings, "origin_type")
        col.prop(settings, "location")
        col.prop(settings, "rotation")
        col.prop(settings, "scale")
        col.prop(settings, "autosmooth_angle")


def register():
    bpy.utils.register_module(__name__)
    bpy.types.Scene.my_tool_settings = PointerProperty(type=PG_MySetings)

def unregister():
    bpy.utils.unregister_module(__name__)
    del bpy.types.Scene.my_tool_settings

if __name__ == '__main__':
    register()